#!/bin/bash
#
# Bash script for DRV8834 stepper motor controler
# Radek Kaczorek <rkaczorek@gmail.com>
# GNU General Public License
#
#
# GPIO mapping
# GPIO4  - DIR
# GPIO17 - STEP
# GPIO22 - M0
# GPIO27 - M1
# GPIO23 - SLEEP 
#
# Steps per revolution
# 360/15 * 76:1 = 1824
#
# Resolution
# 1/1	- M0=0 M1=0
# 1/2	- M0=1 M1=0
# 1/4	- M0=Floating M1=0
# 1/8	- M0=0 M1=1
# 1/16	- M0=1 M1=1
# 1/32	- M0=Floating M1=1

gpio_open () {
	if [ ! -e "/sys/class/gpio/gpio$1/value" ]; then
		echo $1 > /sys/class/gpio/export
		echo out > /sys/class/gpio/gpio$1/direction
		echo "GPIO$1 opened"
	else
		echo "GPIO$1 already opened"
	fi
}

gpio_close () {
	if [ -e "/sys/class/gpio/gpio$1/value" ]; then
		echo $1 > /sys/class/gpio/unexport
		echo "GPIO$1 closed"
	else
		echo "GPIO$1 already closed"
	fi
}
set_dir () {
	case "$1" in
		"R")
			echo 1 > /sys/class/gpio/gpio4/value
		;;
		"L")
			echo 0 > /sys/class/gpio/gpio4/value
		;;
		*)
			echo 1 > /sys/class/gpio/gpio4/value
	esac
}

set_res () {
        case "$1" in
                "1")
                        echo 0 > /sys/class/gpio/gpio22/value
			echo 0 > /sys/class/gpio/gpio27/value
                ;;
                "2")
                        echo 1 > /sys/class/gpio/gpio22/value
			echo 0 > /sys/class/gpio/gpio27/value
                ;;
                "3")
                        echo 0 > /sys/class/gpio/gpio22/value
                        echo 1 > /sys/class/gpio/gpio27/value
                ;;
                "4")
                        echo 1 > /sys/class/gpio/gpio22/value
                        echo 1 > /sys/class/gpio/gpio27/value
                ;;
                *)
                        echo 0 > /sys/class/gpio/gpio22/value
                        echo 0 > /sys/class/gpio/gpio27/value
        esac
}

step () {
	if [ -e "/sys/class/gpio/gpio4/value" ]; then
		set_dir $1
	else
		echo "GPIO4 not opened. Terminating"
		exit 1
	fi
        if [ -e "/sys/class/gpio/gpio22/value" ] && [ -e "/sys/class/gpio/gpio27/value" ]; then
                set_res $2
        else 
                echo "GPIO22 or GPIO27 not opened. Terminating"
		exit 1
        fi
	if [ -e "/sys/class/gpio/gpio17/value" ]; then
		echo 1 > /sys/class/gpio/gpio17/value
		sleep 0.000005
		echo 0 > /sys/class/gpio/gpio17/value
		sleep 0.000005
	else
		echo "GPIO17 not opened. Terminating"
		exit 1
	fi
}

travel () {
	if [ $1 > 0 ]; then
		# wakeup
		echo 1 > /sys/class/gpio/gpio23/value
		# travel
		for (( i=1; i<=$1; i++)); do
			step $2 $3
			#echo "Travelling... $i/$1"
		done
		# sleep
		echo 0 > /sys/class/gpio/gpio23/value
	fi
}

case $1 in
        "open")
		gpio_open 17	#STEP
		gpio_open 4	#DIR
		gpio_open 22	#M0
		gpio_open 27	#M1
		gpio_open 23	#SLEEP
	;;
	"close")
        	gpio_close 17	#STEP
		gpio_close 4	#DIR
		gpio_close 22	#M0
		gpio_close 27	#M1
		gpio_close 23	#SLEEP
	;;
	"travel")
		travel $2 $3 $4 # number_of_steps direction resolution
	;;
	*)
		echo "Usage: motor open | close | travel ( number_of_steps [R|L] [resolution] )"
esac

